@@ -6,8 +6,13 @@
 #' This function can also generate love plots of the same quantities.
 #'
 #' @inheritParams stand
+#' @inheritParams optimize_controls
 #' @param X a data frame containing the covariates in the columns over which balance is desired. The number
 #' of rows should equal the length of \code{z}.
+#' @param treated which treatment value should be considered the treated units. This
+#' must be one of the values of \code{z}.
+#' @param control which treatment value should be considered the control units. This
+#' must be one of the values of \code{z}.
 #' @param selected a boolean vector including whether each unit was selected as part of the treated and control
 #' groups for analysis. Should be the same length as \code{z} and typically comes from the results of
 #' \code{\link{optimize_controls}()}.
@@ -65,13 +70,13 @@
 #'                              selected = results$selected,
 #'                              plot = TRUE)
 
-
-check_balance <- function(z, X, st, selected, denom_variance = "treated", plot = FALSE, message = TRUE) {
+check_balance <- function(z, X, st, selected, treated = 1, control = 0,
+                          denom_variance = "treated", plot = FALSE, message = TRUE) {
 
   if (plot && !requireNamespace("ggplot2", quietly = TRUE) && !requireNamespace("rlang", quietly = TRUE)) {
-      stop("Packages \"ggplot2\" and \"rlang\" needed if \"plot\" argument set to \"TRUE\". Please
+    stop("Packages \"ggplot2\" and \"rlang\" needed if \"plot\" argument set to \"TRUE\". Please
          install these or switch the \"plot\" argument to \"FALSE\".",
-           call. = FALSE)
+         call. = FALSE)
   }
 
   st <- as.factor(st)
@@ -79,26 +84,31 @@ check_balance <- function(z, X, st, selected, denom_variance = "treated", plot =
   dummies <- dummyVars( ~ ., data = X, levelsOnly = FALSE)
   full_X <- predict(dummies, newdata = X)
 
-  sd_across <- get_stand_diffs(full_X, z, selected, denom_variance = denom_variance)
+  sd_across <- get_stand_diffs(full_X, z, selected, treated = treated, control = control,
+                               denom_variance = denom_variance)
 
   sd_strata <- NULL
   for (ist in levels(st)) {
     sd_strata <- rbind(sd_strata, cbind(get_stand_diffs(full_X, z, selected, st, ist,
+                                                        treated = treated, control = control,
                                                         denom_variance = denom_variance), ist))
   }
   colnames(sd_strata)[4] <- "stratum"
 
-  q_s <- sapply(levels(st), function(ist) {sum( !z & selected & st == ist )})
-  n_s <- sapply(levels(st), function(ist) {sum( !z & st == ist )})
+  q_s <- sapply(levels(st), function(ist) {sum( z == control & selected & st == ist )})
+  n_s <- sapply(levels(st), function(ist) {sum( z == control & st == ist )})
 
-  fr_tab <- table(z, st)
   sd_strata_avg <- sd_across
   sd_strata_avg[1:dim(sd_strata_avg)[1], 1:2] <- NA
   for (cov in row.names(sd_strata_avg)) {
     sd_strata_avg[cov, 1] <- sum(sapply(levels(st), function(ist) {
-      sd_strata[sd_strata$covariate == cov & sd_strata$stratum == ist, 1] * (n_s[ist] - sum(is.na(X[!z & st == ist, cov]))) })) / (sum(n_s) - sum(is.na(X[!z, cov])))
+      sd_strata[sd_strata$covariate == cov & sd_strata$stratum == ist, 1] *
+        (n_s[ist] - sum(is.na(X[z == 0 & st == ist, cov]))) })) /
+      (sum(n_s) - sum(is.na(X[z == 0, cov])))
     sd_strata_avg[cov, 2] <- sum(sapply(levels(st), function(ist) {
-      sd_strata[sd_strata$covariate == cov & sd_strata$stratum == ist, 2] * (q_s[ist] - sum(is.na(X[!z & st == ist & selected, cov]))) })) / (sum(q_s) - sum(is.na(X[!z & selected, cov])))
+      sd_strata[sd_strata$covariate == cov & sd_strata$stratum == ist, 2] *
+        (q_s[ist] - sum(is.na(X[z == 0 & st == ist & selected, cov]))) })) /
+      (sum(q_s) - sum(is.na(X[z == 0 & selected, cov])))
   }
 
   if (message) {
@@ -169,41 +179,50 @@ check_balance <- function(z, X, st, selected, denom_variance = "treated", plot =
 #' choosing a subset of controls, and one for after. The rows pertain to covariates.
 #' @keywords internal
 
-get_stand_diffs <- function(data, z, selected, st = NULL, ist = NULL, denom_variance = "treated") {
+get_stand_diffs <- function(data, z, selected, st = NULL, ist = NULL,
+                            treated = 1, control = 0, denom_variance = "treated") {
+  if (is.vector(z)) {
+    z <- as.factor(z)
+  }
   if (!is.null(ist)) {
     ind <- st == ist
   } else {
     ind <- rep(TRUE, length(z))
   }
-  treatedmat_full <- data[z == 1, , drop = FALSE]
-  treatedmat <- data[z == 1 & ind, , drop = FALSE]
   # Standardized differences before matching
-  controlmat_before_full <- data[z == 0, , drop = FALSE]
-  controlmat_before <- data[z == 0 & ind, , drop = FALSE]
+  treatedmat_before_full <- data[z == treated, , drop = FALSE]
+  treatedmat_before <- data[z == treated & ind, , drop = FALSE]
+  treatedmean_before <- apply(treatedmat_before, 2, mean, na.rm = TRUE)
+  controlmat_before_full <- data[z == control, , drop = FALSE]
+  controlmat_before <- data[z == control & ind, , drop = FALSE]
   controlmean_before <- apply(controlmat_before, 2, mean, na.rm = TRUE)
-  treatmean <- apply(treatedmat, 2, mean, na.rm = TRUE)
-  treatvar <- apply(treatedmat_full, 2, var, na.rm = TRUE)
-  controlvar <- apply(controlmat_before_full, 2, var, na.rm = TRUE)
-  if (dim(treatedmat_full)[1] == 1) {
-    treatvar[1:length(treatvar)] <- 0.0
+  variances <- sapply(levels(z), function(group) {
+    return(apply(data[z == group, , drop = FALSE], 2, var, na.rm = TRUE))
+  })
+  if (is.vector(variances)) {
+    variances <- matrix(variances, ncol = 1)
   }
+  variances[is.na(variances)] <- 0
   if (denom_variance == "pooled") {
-    denom <- sqrt((treatvar + controlvar) / 2)
+    denom <- sqrt(rowMeans(variances))
   } else {
-    denom <- sqrt(treatvar)
-    denom[treatvar == 0] <- sqrt(controlvar[treatvar == 0] / 2)
+    denom <- sqrt(variances[, levels(z) == treated])
+    denom[denom == 0] <-
+      sqrt(rowMeans(variances)[denom == 0])
   }
-  stand_diff_before <- rep(NA, length(treatvar))
-  names(stand_diff_before) <- names(treatvar)
-  stand_diff_before <- (treatmean - controlmean_before) / denom
-  stand_diff_before[treatmean == controlmean_before] <- 0.0
+  stand_diff_before <- rep(NA, nrow(variances))
+  names(stand_diff_before) <- dimnames(variances)[[1]]
+  stand_diff_before <- (treatedmean_before - controlmean_before) / denom
+  stand_diff_before[treatedmean_before == controlmean_before] <- 0.0
   # Standardized differences after matching
-  controlmat_after <- data[selected & z == 0 & ind, , drop = FALSE]
+  controlmat_after <- data[selected & z == control & ind, , drop = FALSE]
   controlmean_after <- apply(controlmat_after, 2, mean, na.rm = TRUE)
-  stand_diff_after <- rep(NA, length(treatvar))
-  names(stand_diff_after) <- names(treatvar)
-  stand_diff_after <- (treatmean - controlmean_after) / denom
-  stand_diff_after[treatmean == controlmean_after] <- 0.0
+  treatedmat_after <- data[selected & z == treated & ind, , drop = FALSE]
+  treatedmean_after <- apply(treatedmat_after, 2, mean, na.rm = TRUE)
+  stand_diff_after <- rep(NA, nrow(variances))
+  names(stand_diff_after) <- dimnames(variances)[[1]]
+  stand_diff_after <- (treatedmean_after - controlmean_after) / denom
+  stand_diff_after[treatedmean_after == controlmean_after] <- 0.0
   sd_matrix <- data.frame(abs_stand_diff_before = abs(stand_diff_before),
                           abs_stand_diff_after = abs(stand_diff_after))
   if (!is.null(ist)) {
@@ -329,8 +348,8 @@ plot_stand_diffs <- function(sds, type) {
                                  stratum = sds$sd_strata$stratum)
 
     p <- apply(as.array(unique(sds$sd_strata$stratum)), 1, function(x) {
-        ggplot(plot_dataframe[plot_dataframe$stratum == x,],
-                      aes(x = .data$abs_stand_diff, y = .data$covariates)) +
+      ggplot(plot_dataframe[plot_dataframe$stratum == x,],
+             aes(x = .data$abs_stand_diff, y = .data$covariates)) +
         geom_point(size = 5, aes(shape = .data$type)) +
         scale_shape_manual(values = c(4, 1)) +
         geom_vline(xintercept = c(.1,.2), lty = 2) +
